"""
Author: Michel Peltriaux
Organization: Struktur- und Genehmigungsdirektion Nord, Rhineland-Palatinate, Germany
Contact: michel.peltriaux@sgdnord.rlp.de
Created on: 01.12.20

"""
from django.http import HttpRequest
from django.urls import reverse
from django.utils.html import format_html
from django.utils.timezone import localtime
from django.utils.translation import gettext_lazy as _

from compensation.filters import CompensationTableFilter
from compensation.models import Compensation
from intervention.filters import InterventionTableFilter
from konova.sub_settings.django_settings import DEFAULT_DATE_TIME_FORMAT
from konova.utils.tables import BaseTable
import django_tables2 as tables


class CompensationTable(BaseTable):
    id = tables.Column(
        verbose_name=_("Identifier"),
        orderable=True,
        accessor="identifier",
    )
    t = tables.Column(
        verbose_name=_("Title"),
        orderable=True,
        accessor="title",
    )
    c = tables.Column(
        verbose_name=_("Checked"),
        orderable=True,
        empty_values=[],
        accessor="intervention__checked",
    )
    r = tables.Column(
        verbose_name=_("Recorded"),
        orderable=True,
        empty_values=[],
        accessor="intervention__recorded",
    )
    e = tables.Column(
        verbose_name=_("Editable"),
        orderable=True,
        empty_values=[],
        accessor="intervention__users",
    )
    lm = tables.Column(
        verbose_name=_("Last edit"),
        orderable=True,
        accessor="created__timestamp",
    )

    class Meta(BaseTable.Meta):
        template_name = "django_tables2/bootstrap4.html"

    def __init__(self, request:HttpRequest, *args, **kwargs):
        self.title = _("Compensations")
        self.add_new_url = reverse("compensation:new")
        qs = kwargs.get("queryset", None)
        self.filter = CompensationTableFilter(
            user=request.user,
            data=request.GET,
            queryset=qs,
        )
        super().__init__(request, self.filter, *args, **kwargs)

    def render_id(self, value, record: Compensation):
        """ Renders the id column for a compensation

        Args:
            value (str): The identifier value
            record (Compensation): The compensation record

        Returns:

        """
        html = ""
        html += self.render_link(
            tooltip=_("Open {}").format(_("Compensation")),
            href=reverse("compensation:open", args=(record.id,)),
            txt=value,
            new_tab=False,
        )
        return format_html(html)

    def render_c(self, value, record: Compensation):
        """ Renders the checked column for a compensation

        checked is set by the main object Intervention

        Args:
            value (str): The identifier value
            record (Compensation): The compensation record

        Returns:

        """
        html = ""
        checked = value is not None
        tooltip = _("Not checked yet")
        if checked:
            value = value.timestamp
            value = localtime(value)
            checked_on = value.strftime(DEFAULT_DATE_TIME_FORMAT)
            tooltip = _("Checked on {} by {}").format(checked_on, record.intervention.checked.user)
        html += self.render_checked_star(
            tooltip=tooltip,
            icn_filled=checked,
        )
        return format_html(html)

    def render_r(self, value, record: Compensation):
        """ Renders the registered column for a compensation

        Args:
            value (str): The identifier value
            record (Compensation): The compensation record

        Returns:

        """
        html = ""
        checked = value is not None
        tooltip = _("Not recorded yet")
        if checked:
            value = value.timestamp
            value = localtime(value)
            on = value.strftime(DEFAULT_DATE_TIME_FORMAT)
            tooltip = _("Recorded on {} by {}").format(on, record.intervention.recorded.user)
        html += self.render_bookmark(
            tooltip=tooltip,
            icn_filled=checked,
        )
        return format_html(html)

    def render_e(self, value, record: Compensation):
        """ Renders the registered column for a compensation

        Args:
            value (str): The identifier value
            record (Compensation): The compensation record

        Returns:

        """
        html = ""
        has_access = value.filter(
            username=self.user.username
        ).exists()

        html += self.render_icn(
            tooltip=_("Full access granted") if has_access else _("Access not granted"),
            icn_class="fas fa-edit rlp-r-inv" if has_access else "far fa-edit",
        )
        return format_html(html)


class EcoAccountTable(BaseTable):
    id = tables.Column(
        verbose_name=_("Identifier"),
        orderable=True,
        accessor="identifier",
    )
    t = tables.Column(
        verbose_name=_("Title"),
        orderable=True,
        accessor="title",
    )
    d = tables.Column(
        verbose_name=_("Created on"),
        orderable=True,
        accessor="created__timestamp",
    )
    ac = tables.Column(
        verbose_name=_("Actions"),
        orderable=False,
        empty_values=[],
        attrs={"td": {"class": "action-col"}}
    )

    class Meta(BaseTable.Meta):
        pass

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.title = _("Eco Accounts")
        self.add_new_url = reverse("compensation:acc-new")

    def render_ac(self, value, record):
        """
        Renders possible actions for this record, such as delete.
        """
        intervention = _("Compensation")
        html = ""
        html += self.render_open_btn(
            _("Open {}").format(intervention),
            reverse("compensation:open", args=(record.id,)),
            new_tab=True
        )
        html += self.render_edit_btn(
            _("Edit {}").format(intervention),
            reverse("compensation:edit", args=(record.id,)),
        )
        html += self.render_delete_btn(
            _("Delete {}").format(intervention),
            reverse("compensation:remove", args=(record.id,)),
        )
        return format_html(html)
